//输换


var fs = require('fs'),
    path = require('path'),
    xml2js = require('../xml2js/lib/xml2js'),
    decode = require('decode-html');
var debug = require('debug')('wevue');
var tagNames = {
    'view': 'div',
    'form': 'form',
    'input': 'input',
    'button': 'button',
    'picker': 'picker',
    'image': 'img',
    'text': 'span'
};

//必转
var attrNames = {
    'wx:if': 'v-if',
    'wx:else': 'v-else',
    'wx:elif': 'v-else-if',
    'wx:for': 'v-for',
    'wx:for-item': 'v-for-item',
    'wx:for-index': 'v-for-index',
    'wx:key': 'v-key'
}


function tagNameProcessors(name) {
    var name = name.toLowerCase();
    debug('tag', name);
    return tagNames[name] || name;
}

function valueProcessors(value) {
    return value;
}

//处理属性名称
function attrNameProcessors(value, name, node) {
    var name = name.toLowerCase();
    debug('value=', value, 'name=', name);
    try {
        if (attrNames[name]) {
            return attrNames[name];
        } else if (/{{.*}}/.test(value)) {
            return 'v-bind:' + name;
        } else {
            return name;
        }
    } catch (e) {
        console.log(e.stack);
    }
}

//处理属性值
//src="abc{{value}}"   =>   v-bind:src="  'abc' + value"
//src="{{value}}abc"   =>   v-bind:src="   value+'abc'"
//src="abc{{value}}efg"   =>   v-bind:src=" 'abc' + value+'efg'"
function attrValueProcessors(value, name, node) {
    var name = name.toLowerCase();
    try {
        debug('attrValue', name, value);
        if (!value) {
            return value;
        }
        if (/{{.*}}/.test(value)) {
            //处理带绑定属性值
            var matches = value.match(/(.*){{(.*)}}(.*)/);
            debug(matches, matches.length);
            if (matches && matches.length === 4) {
                var left = matches[1];
                var exp = matches[2];
                var right = matches[3];
                if (left && exp && right) {
                    value = `'${left}'+ ${exp} +'${right}'`;
                } else if (left && exp) {
                    value = `'${left}'+ ${exp}`;
                } else if (right) {
                    value = `${exp} +'${right}'`;
                } else {
                    value = exp;
                }
            }
        }

        var ret = "";
        switch (name) {
            case 'wx:for':
                var item = node.attributes['wx:for-item'.toUpperCase()];
                var index = node.attributes['wx:for-index'.toUpperCase()];
                if (item && index) {
                    ret = `(${index},${item}) in ${value}`;
                } else if (item) {
                    ret = `${item} in ${value}`;
                } else if (index) {
                    ret = `(${index},item) in ${value}`;
                } else {
                    ret = `for ${item} in ${value}`;
                }
                break;
            case 'wx:for-index':
            case 'wx:for-item':
            case 'wx:key':
                break;
            default:
                ret = value;
        }
    } catch (e) {
        console.log(e.stack);
    }
    return ret;
}

function parser(wxml, cb) {

    var parseString = xml2js.parseString;
    var builder = new xml2js.Builder();

    parseString('<root>' + wxml + '</root>', {
        tagNameProcessors: [tagNameProcessors],
        attrNameProcessors: [attrNameProcessors],
        valueProcessors: [valueProcessors],
        attrValueProcessors: [attrValueProcessors],
        strict: false,
        normalize: true
    }, function(err, result) {
        var xml;
        if (result) {
            xml = builder.buildObject(result).replace('<?xml version="1.0" encoding="UTF-8" standalone="yes"?>', '');
            debug('xml 1',xml);
            xml = xml.substr(7, xml.length - 14);
            debug('xml 2',xml);
        }
        return cb(err, xml);
    });
}

module.exports = parser;
